/// <reference types="codemirror" />
import { UserConfig } from 'emmet';
import { TextRange } from '@emmetio/action-utils';
import { AbbrError } from '../lib/utils';
interface AbbrBase {
    abbr: string;
    type: string;
}
interface ParsedAbbreviation extends AbbrBase {
    type: 'abbreviation';
    simple: boolean;
    preview: string;
}
interface ParsedAbbreviationError extends AbbrBase {
    type: 'error';
    error: AbbrError;
}
export interface StartTrackingParams {
    options: UserConfig;
    offset?: number;
    forced?: boolean;
}
interface StopTrackingOptions {
    /** Do not remove contents of force-tracked abbreviation */
    skipRemove?: boolean;
    /** Forced tracker remove, do not add it to history */
    force?: boolean;
}
export interface SerializedTracker {
    range: TextRange;
    abbr?: string;
    valid: boolean;
    forced: boolean;
    offset: number;
}
export default class AbbreviationTracker {
    /** Last caret location in document */
    lastPos: number;
    /** Last document length */
    lastLength: number;
    /** Current abbreviation range */
    range: TextRange;
    /** Offset in range where abbreviation actually starts */
    offset: number;
    /** Abbreviation is forced, e.g. must be kept in editor as long as possible */
    forced: boolean;
    /** Parsed abbreviation for current range. May contain error */
    abbreviation: ParsedAbbreviation | ParsedAbbreviationError | null;
    options: UserConfig;
    private marker;
    private preview;
    private forcedMarker;
    constructor(start: number, pos: number, length: number, params: StartTrackingParams);
    /**
     * Marks tracker in given editor
     */
    mark(editor: CodeMirror.Editor): void;
    /**
     * Remove current tracker marker
     */
    unmark(): void;
    showPreview(editor: CodeMirror.Editor): void;
    /**
     * Removes preview of current abbreviation
     */
    hidePreview(): void;
    /**
     * Check if current range contains given position
     */
    contains(pos: number): boolean;
    /**
     * Returns serialized copy of current tracked
     */
    serialize(): SerializedTracker;
    /**
     * Stores contents of current tracker in internal state of given editor.
     * Stored tracker can be restored later
     */
    save(editor: CodeMirror.Editor): void;
    private disposeMarker;
}
/**
 * Returns abbreviation tracker for given editor
 */
export declare function getTracker(editor: CodeMirror.Editor): AbbreviationTracker | undefined;
/**
 * Starts abbreviation tracking for given editor
 * @param start Location of abbreviation start
 * @param pos Current caret position, must be greater that `start`
 */
export declare function startTracking(editor: CodeMirror.Editor, start: number, pos: number, params?: Partial<StartTrackingParams>): AbbreviationTracker;
/**
 * Stops abbreviation tracking in given editor instance
 */
export declare function stopTracking(editor: CodeMirror.Editor, options?: StopTrackingOptions): void;
/**
 * Handle content change in given editor instance
 */
export declare function handleChange(editor: CodeMirror.Editor, pos: number): AbbreviationTracker | undefined;
export declare function handleSelectionChange(editor: CodeMirror.Editor, pos: number): AbbreviationTracker | undefined;
/**
 * Tries to restore abbreviation tracker for given editor at specified position
 */
export declare function restoreTracker(editor: CodeMirror.Editor, pos: number): AbbreviationTracker | undefined;
export {};
